<?php

namespace Drupal\corona_counter\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Locale\CountryManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'CoronaCounterBlock' block.
 *
 * @Block(
 *  id = "corona_counter_block",
 *  admin_label = @Translation("Corona Counter"),
 * )
 */
class CoronaCounterBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Gets the statistics of all countries.
   */
  const ALL_API = 'https://corona.lmao.ninja/v3/covid-19/all';

  /**
   * Gets the statistics of a country.
   */
  const COUNTRY_STATISTICS_API = 'https://corona.lmao.ninja/v3/covid-19/countries/%s';

  /**
   * Guzzle\Client instance.
   *
   * @var \Guzzle\Client
   */
  protected $httpClient;

  /**
   * The country manager
   *
   * @var \Drupal\Core\Locale\CountryManagerInterface
   */
  protected $countryManager;

  /**
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Guzzle\Client $httpClient
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Client $httpClient, CountryManagerInterface $country_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->httpClient = $httpClient;
    $this->countryManager = $country_manager;
  }

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   *
   * @return static
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
      $container->get('country_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'countries' => [],
      'all_countries' => TRUE,
      'cache_max_age' => 300,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return $this->configuration['cache_max_age'] ?? 0;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $countries = $this->countryManager->getList();
    $form['countries'] = [
      '#type' => 'select',
      '#title' => $this->t('Countries'),
      '#description' => $this->t('The countries statistics will be shown below the global statistics.'),
      '#options' => $countries,
      '#default_value' => $this->configuration['countries'],
      '#multiple' => TRUE,
    ];

    $form['cache_max_age'] = [
      '#type' => 'number',
      '#title' => $this->t('Max Age'),
      '#description' => $this->t('Set the cache max age in seconds'),
      '#default_value' => $this->configuration['cache_max_age'],
    ];

    $form['show_all_countries'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show global statistics'),
      '#default_value' => $this->configuration['show_all_countries'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['countries'] = $form_state->getValue('countries');
    $this->configuration['show_all_countries'] = $form_state->getValue('show_all_countries');
  }

  /**
   * {@inheritdoc}
   *
   * @return array
   *   A render array used to show country and cases
   */
  public function build() {
    try {
      if ($this->configuration['show_all_countries']) {
        $request = $this->httpClient->request('GET', self::ALL_API);
        $global_stats = json_decode($request->getBody()->getContents(), TRUE);
        $data['all_countries'] = $global_stats;
      }

      $countries = $this->configuration['countries'];
      if (!empty($countries)) {
        $countries_query = implode(',', $countries);
        $request = $this->httpClient->request('GET', sprintf(self::COUNTRY_STATISTICS_API, $countries_query));
        $countries_statistics = json_decode($request->getBody()->getContents(), TRUE);

        // If we request for one country, the result is not an of countries, we normalize it into an array.
        if (count($countries) == 1) {
          $countries_statistics = [$countries_statistics];
        }

        foreach ($countries_statistics as $country) {
          // We added a 'name' for backward compatibility with the older version.
          $data[$country['countryInfo']['iso2']] = ['name' => $country['country']] + $country;
        }
      }

      return [
        '#theme' => 'corona_counter_block',
        '#content' => $data,
        '#attached' => [
          'library' => [
            'corona_counter/block_style',
          ],
        ],
      ];
    } catch (GuzzleException $e) {
      return [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $this->t("Something unexpected happened, please contact the administrators if the problem presists"),
      ];
    }
  }
}
